/*
 * Copyright (c) 2015 NIBIO <http://www.nibio.no/>. 
 * 
 * This file is part of RoughageNutritionModel.
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 * 
 */
package no.bioforsk.vips.model.roughagenutritionmodel;

import java.util.Date;

/**
 * Dataobjekt for lagring av inndata for optimeringsrutinene
 * &copy; 2009-2015 NIBIO
 * @author Tor-Einar Skog <tor-einar.skog@nibio.no>
 */
public class OptimizationObservation implements Comparable{

    private Date date;
    private Double height;
    private Double MSC;
    private Double NDF;
    private Double INDF;
    private Double rawProtein;
    private Double FEm;

    public OptimizationObservation()
    {
        
    }
    
    public OptimizationObservation(
            Date date,
            Double height,
            Double MSC,
            Double NDF,
            Double INDF,
            Double rawProtein,
            Double FEm
    )
    {
        this.date = date;
        this.height = height;
        this.MSC = MSC;
        this.NDF = NDF;
        this.INDF = INDF;
        this.rawProtein = rawProtein;
        this.FEm = FEm;
    }

    /**
     * Bruker formelen Avling (kg TS/daa) = 34.86 * 0.017h<sup>2</sup> + 10.21h
     * @return
     */
    public Double getAvling()
    {
        if(this.getHeight() == null)
        {
            return null;
        }
        return 34.86f + 0.017f * (Double) Math.pow(this.getHeight(), 2) + 10.21f * this.getHeight();
    }

    @Override
    public int compareTo(Object o) {
        if(! (o instanceof OptimizationObservation))
            return 0;
        else
            return this.compareTo((OptimizationObservation) o);
    }

    public int compareTo(OptimizationObservation other)
    {
        return this.getDate().compareTo(other.getDate());
    }

    /**
     * @return the date
     */
    public Date getDate() {
        return date;
    }

    /**
     * @param date the date to set
     */
    public void setDate(Date date) {
        this.date = date;
    }

    /**
     * @return the height
     */
    public Double getHeight() {
        return height;
    }

    /**
     * @param height the height to set
     */
    public void setHeight(Double height) {
        this.height = height;
    }

    /**
     * @return the MSC
     */
    public Double getMSC() {
        return MSC;
    }

    /**
     * @param MSC the MSC to set
     */
    public void setMSC(Double MSC) {
        this.MSC = MSC;
    }

    /**
     * @return the NDF
     */
    public Double getNDF() {
        return NDF;
    }

    /**
     * @param NDF the NDF to set
     */
    public void setNDF(Double NDF) {
        this.NDF = NDF;
    }

    /**
     * @return the INDF
     */
    public Double getINDF() {
        return INDF;
    }

    /**
     * @param INDF the INDF to set
     */
    public void setINDF(Double INDF) {
        this.INDF = INDF;
    }

    /**
     * @return the rawProtein
     */
    public Double getRawProtein() {
        return rawProtein;
    }

    /**
     * @param rawProtein the rawProtein to set
     */
    public void setRawProtein(Double rawProtein) {
        this.rawProtein = rawProtein;
    }

    /**
     * @return the FEm
     */
    public Double getFEm() {
        return FEm;
    }

    /**
     * @param FEm the FEm to set
     */
    public void setFEm(Double FEm) {
        this.FEm = FEm;
    }

    @Override
    public String toString(){
        return "OptimizationObservation[" + this.getDate() + "] "
                + "Yield = " + this.getAvling() 
                + ", FEm = " + this.getFEm()
                + ", Height = " + this.getHeight()
                + ", INDF = " + this.getINDF()
                + ", MSC = " + this.getMSC()
                + ", NDF = " + this.getNDF()
                + ", RawProtein = " + this.getRawProtein();
    }
}
