/*
 * Copyright (c) 2015 NIBIO <http://www.nibio.no/>. 
 * 
 * This file is part of RoughageNutritionModel.
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 * 
 */

package no.bioforsk.vips.model.roughagenutritionmodel;

import java.text.DecimalFormat;
import java.util.Date;
import java.util.HashMap;
import no.nibio.vips.util.DateMap;

/**
 * Matrisedataobjekt som lagrer alle resultat fra beregning av grovformodellen
 * <br/>
 * &copy; 2009-2015 NIBIO
 * @author Tor-Einar Skog <tor-einar.skog@nibio.no>
 */
public class ResultMatrix extends DateMap{
    //public final static String V = "V";
    public final static String AVLING = "Avling";
    private final String AVLING_DECIMALFORMAT_PATTERN = "###";
    //public final static String DVR = "DVR";
    public final static String MSC_BEREGNET = "Beregnet MSC";
    private final String MSC_DECIMALFORMAT_PATTERN = "###.##";
    public final static String NDF = "NDF";
    private final String NDF_DECIMALFORMAT_PATTERN = "###.#";
    public final static String INDF = "INDF";
    private final String INDF_DECIMALFORMAT_PATTERN = "###.#";
    public final static String RAAPROTEIN = "R&aring;protein";
    private final String RAAPROTEIN_DECIMALFORMAT_PATTERN = "###.#";
    public final static String FEM = "FEm";
    private final String FEM_DECIMALFORMAT_PATTERN = "###.##";

    private final String DEFAULT_DECIMALFORMAT_PATTERN ="###.##";

    private HashMap<String, DecimalFormat> decimalFormats;

    // Optimerte parametre
    private boolean optimeringForetatt = false;
    private Double alfa;
    private Double RmFoersteslaatt;
    private Double RmAndreslaatt;
    private Double NRmFoersteslaatt;
    private Double NRmAndreslaatt;
    private Double cFoersteslaatt;
    private Double IRmFoersteslaatt;
    //private Float betaINDF;
    private Double pINDF;
    private Double FEmj;
    private Double FEmm;

    /** Creates a new instance of GrovforModellResultatMatriseBO */
    public ResultMatrix()
    {
        super();
        initDecimalFormats();
    }

    private void initDecimalFormats()
    {
        this.decimalFormats = new HashMap();
        this.decimalFormats.put(ResultMatrix.AVLING, new DecimalFormat(this.AVLING_DECIMALFORMAT_PATTERN));
        this.decimalFormats.put(ResultMatrix.FEM, new DecimalFormat(this.FEM_DECIMALFORMAT_PATTERN));
        this.decimalFormats.put(ResultMatrix.INDF, new DecimalFormat(this.INDF_DECIMALFORMAT_PATTERN));
        this.decimalFormats.put(ResultMatrix.MSC_BEREGNET, new DecimalFormat(this.MSC_DECIMALFORMAT_PATTERN));
        this.decimalFormats.put(ResultMatrix.NDF, new DecimalFormat(this.NDF_DECIMALFORMAT_PATTERN));
        this.decimalFormats.put(ResultMatrix.RAAPROTEIN, new DecimalFormat(this.RAAPROTEIN_DECIMALFORMAT_PATTERN));
        this.decimalFormats.put("DEFAULT", new DecimalFormat(this.DEFAULT_DECIMALFORMAT_PATTERN));
    }

    private DecimalFormat getDecimalFormat(String paramName)
    {
        return this.decimalFormats.get(paramName) != null ? this.decimalFormats.get(paramName) : this.decimalFormats.get("DEFAULT");
    }

    /**
     * Returns a formatted string representing the number
     * The format is decided internally in this object
     * @param date
     * @param paramName
     * @return
     */
    public String getDefaultFormattedDoubleValue(Date date, String paramName)
    {
        Double value = this.getParamDoubleValueForDate(date, paramName);
        return this.getDecimalFormat(paramName).format(value);
    }

    /**
     * @return the alfa
     */
    public Double getAlfa() {
        return alfa;
    }

    /**
     * @param alfa the alfa to set
     */
    public void setAlfa(Double alfa) {
        this.alfa = alfa;
    }

    /**
     * @return the RmFoersteslaatt
     */
    public Double getRmFoersteslaatt() {
        return RmFoersteslaatt;
    }

    /**
     * @param RmFoersteslaatt the RmFoersteslaatt to set
     */
    public void setRmFoersteslaatt(Double RmFoersteslaatt) {
        this.RmFoersteslaatt = RmFoersteslaatt;
    }

    /**
     * @return the RmAndreslaatt
     */
    public Double getRmAndreslaatt() {
        return RmAndreslaatt;
    }

    /**
     * @param RmAndreslaatt the RmAndreslaatt to set
     */
    public void setRmAndreslaatt(Double RmAndreslaatt) {
        this.RmAndreslaatt = RmAndreslaatt;
    }

    /**
     * @return the NRmFoersteslaatt
     */
    public Double getNRmFoersteslaatt() {
        return NRmFoersteslaatt;
    }

    /**
     * @param NRmFoersteslaatt the NRmFoersteslaatt to set
     */
    public void setNRmFoersteslaatt(Double NRmFoersteslaatt) {
        this.NRmFoersteslaatt = NRmFoersteslaatt;
    }

    /**
     * @return the NRmAndreslaatt
     */
    public Double getNRmAndreslaatt() {
        return NRmAndreslaatt;
    }

    /**
     * @param NRmAndreslaatt the NRmAndreslaatt to set
     */
    public void setNRmAndreslaatt(Double NRmAndreslaatt) {
        this.NRmAndreslaatt = NRmAndreslaatt;
    }

    /**
     * @return the cFoersteslaatt
     */
    public Double getCFoersteslaatt() {
        return cFoersteslaatt;
    }

    /**
     * @param cFoersteslaatt the cFoersteslaatt to set
     */
    public void setCFoersteslaatt(Double cFoersteslaatt) {
        this.cFoersteslaatt = cFoersteslaatt;
    }

    /**
     * @return the IRmFoersteslaatt
     */
    public Double getIRmFoersteslaatt() {
        return IRmFoersteslaatt;
    }

    /**
     * @param IRmFoersteslaatt the IRmFoersteslaatt to set
     */
    public void setIRmFoersteslaatt(Double IRmFoersteslaatt) {
        this.IRmFoersteslaatt = IRmFoersteslaatt;
    }

    /**
     * @return the betaINDF
     *
    public Float getBetaINDF() {
        return betaINDF;
    }

    /**
     * @param betaINDF the betaINDF to set
     *
    public void setBetaINDF(Float betaINDF) {
        this.betaINDF = betaINDF;
    }*/

    /**
     * @return the optimeringForetatt
     */
    public boolean isOptimeringForetatt() {
        return optimeringForetatt;
    }

    /**
     * @param optimeringForetatt the optimeringForetatt to set
     */
    public void setOptimeringForetatt(boolean optimeringForetatt) {
        this.optimeringForetatt = optimeringForetatt;
    }

    /**
     * @return the FEmj
     */
    public Double getFEmj() {
        return FEmj;
    }

    /**
     * @param FEmj the FEmj to set
     */
    public void setFEmj(Double FEmj) {
        this.FEmj = FEmj;
    }

    /**
     * @return the pINDF
     */
    public Double getpINDF() {
        return pINDF;
    }

    /**
     * @param pINDF the pINDF to set
     */
    public void setpINDF(Double pINDF) {
        this.pINDF = pINDF;
    }

    /**
     * @return the FEmm
     */
    public Double getFEmm() {
        return FEmm;
    }

    /**
     * @param FEmm the FEmm to set
     */
    public void setFEmm(Double FEmm) {
        this.FEmm = FEmm;
    }
}
