/*
 * Copyright (c) 2015 NIBIO <http://www.nibio.no/>. 
 * 
 * This file is part of RoughageNutritionModel.
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 * 
 */
package no.bioforsk.vips.model.roughagenutritionmodel;

import com.fasterxml.jackson.core.JsonFactory;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.MappingJsonFactory;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ibm.icu.util.Calendar;
import com.ibm.icu.util.TimeZone;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import no.nibio.vips.entity.ModelConfiguration;
import no.nibio.vips.entity.Result;
import no.nibio.vips.entity.WeatherObservation;
import no.nibio.vips.model.ConfigValidationException;
import no.nibio.vips.model.ModelExcecutionException;
import no.nibio.vips.model.ModelId;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Tor-Einar Skog <tor-einar.skog@nibio.no>
 */
public class RoughageNutritionModelTest {
    
    public RoughageNutritionModelTest() {
    }
    
    @BeforeClass
    public static void setUpClass() {
    }
    
    @AfterClass
    public static void tearDownClass() {
    }
    
    @Before
    public void setUp() {
    }
    
    @After
    public void tearDown() {
    }

    /**
     * Test of getResult method, of class RoughageNutritionModel.
     */
    @Test
    public void testGetResult() throws Exception {
        System.out.println("getResult");
        try
        {
            ModelConfiguration config = this.getConfiguration("/weatherdata_kvithamar_2016.json");
            Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("Europe/Oslo"));
            cal.set(2016, Calendar.JUNE, 1, 0, 0, 0);
            cal.set(Calendar.MILLISECOND,0);
            config.setConfigParameter("timeZone","Europe/Oslo");
            config.setConfigParameter("firstHarvest", cal.getTime());
            cal.set(Calendar.MONTH, Calendar.APRIL);
            cal.set(Calendar.DATE, 15);
            config.setConfigParameter("firstPossibleGrowthStartDate", cal.getTime());
            config.setConfigParameter("soilType", RoughageNutritionModel.JORDTYPE_SAND);
            config.setConfigParameter("cloverShare", RoughageNutritionModel.ENGSAMMENSETNING_KLOEVERANDEL_LITEN);
            String[] optimizationInfoLines = {"2016-06-02,2,3,,,,"};
            List<String> optimizationInfo = Arrays.asList(optimizationInfoLines);
            config.setConfigParameter("optimizationInfo", optimizationInfo);
            RoughageNutritionModel instance = new RoughageNutritionModel();
            instance.setConfiguration(config);
            //List<Result> expResult = null;
            List<Result> result = instance.getResult();
            //assertEquals(expResult, result);
            assertNotNull(result);
            Collections.sort(result);
            //System.out.println("Number of result objects: " + result.size());
            for(Result r:result)
            {
                System.out.println(r);
            }
        }
        catch(ConfigValidationException | ModelExcecutionException ex)
        {
            ex.printStackTrace();
            fail(ex.getMessage());
        }
    }

    /**
    

    /**
     * Test of getModelId method, of class RoughageNutritionModel.
     */
    @Test
    public void testGetModelId() {
        System.out.println("getModelId");
        RoughageNutritionModel instance = new RoughageNutritionModel();
        ModelId expResult = new ModelId( "ROUGHAGENU");
        ModelId result = instance.getModelId();
        assertEquals(expResult.toString(), result.toString());
        
    }

  
    /**
     * Test of setConfiguration method, of class RoughageNutritionModel.
     */
    @Test
    public void testSetConfiguration() throws Exception {
        System.out.println("setConfiguration");
        try
        {
            ModelConfiguration config = this.getConfiguration("/weatherdata_boe_2015.json");
            Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("Europe/Oslo"));
            cal.set(2015, Calendar.JUNE, 1, 0, 0, 0);
            cal.set(Calendar.MILLISECOND,0);
            config.setConfigParameter("firstHarvest", cal.getTime());
            config.setConfigParameter("soilType", RoughageNutritionModel.JORDTYPE_SAND);
            config.setConfigParameter("cloverShare", RoughageNutritionModel.ENGSAMMENSETNING_KLOEVERANDEL_LITEN);
            RoughageNutritionModel instance = new RoughageNutritionModel();
            instance.setConfiguration(config);
        }
        catch(ConfigValidationException ex)
        {
            fail(ex.getMessage());
        }
    }
    
    private ModelConfiguration getConfiguration(String fileName)
    {
        try {
            ModelConfiguration config = new ModelConfiguration();
            config.setModelId(RoughageNutritionModel.MODEL_ID.toString());
            config.setConfigParameter("timeZone", "Europe/Oslo");
            BufferedInputStream inputStream = new BufferedInputStream(this.getClass().getResourceAsStream(fileName));
            JsonFactory f = new MappingJsonFactory();
            JsonParser jp = f.createParser(inputStream);
            JsonNode all = jp.readValueAsTree();
            //List<WeatherObservation> observations = new ArrayList<>();
            ObjectMapper mapper = new ObjectMapper();
            List<WeatherObservation> observations = mapper.convertValue(all, new TypeReference<List<WeatherObservation>>(){});

            
            config.setConfigParameter("observations", observations);

            
            return config;
        } catch (IOException ex) {
            ex.printStackTrace();
            return null;
        } 
    }
}
